//////////////////////////////////////////////////////////////////////////////////
// $Source: /usr/cvsroot/cdx/src/cdx/cdxsound.h,v $
// $Author: Mindcry $
//
// $Log: cdxsound.h,v $
// Revision 2.9  1999/09/24 10:38:10  Mindcry
// - fixed crashbug with running on NT
// - fixed compiler error with DX7
//
// Revision 2.8  1999/08/29 22:29:04  MICHAELR
// added CDXDLL support to classes/etc
//
// Revision 2.7  1999/08/19 13:54:24  Mindcry
// Added the new function: DWORD Playing( void )
//
// Revision 2.6  1999/07/16 20:40:24  Mindcry
// better checking for correct pointers/data etc.
//
// Revision 2.5  1999/07/16 16:59:33  Mindcry
// Some code changes and another new Load function (for [WAV] Resource)
//
// Revision 2.4  1999/07/16 14:19:26  Mindcry
// * + Added New Load function. Now you can load a Wave from Filepointer
// * ! fixed another strange bug in the Load functions
//
// Revision 2.3  1999/07/16 06:09:06  Mindcry
// Added function to turn sound on/off
//
// Revision 2.2  1999/07/15 16:33:29  Mindcry
// Added Volume Control and 2 Variables (m_NoSoundCard,m_SoundOff)
//
// Revision 2.1  1999/05/29 21:24:25  janni
// (Masturdul) added
// BOOL PlayFrequency(DWORD freq, int Pan=0, DWORD dwFlags=0)
// I have modified the Play function to restore the original
// freq to the sample.
// BOOL Extract(CDXSound* pDS, char *ptr, int Num = 1);
// (i dint find a name in english) That works equal as Load and
// the only diferenceis that Extract recieves a ptr to a place in memory where a WAV
// file is already loaded instead of a filename.
// The new functions associated are:
// BOOL LoadFromMemory(char *ptr);
// BOOL ReadDataFromMemory(char *ptr, DWORD dwSize);
//
// Revision 2.0  1999/05/01 13:51:17  bsimser
// Updated revision number to 2.0
//
// Revision 1.1.1.1  1999/05/01 04:10:56  bsimser
// Initial revision to cvs
//
// $Revision: 2.9 $
//////////////////////////////////////////////////////////////////////////////////
#ifndef CDXSOUND_H
#define CDXSOUND_H

// Fixes the compile problem with DirectX7 SDK
#ifndef DSBCAPS_CTRLDEFAULT
	#define DSBCAPS_CTRLDEFAULT DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME | DSBCAPS_CTRLFREQUENCY
#endif

//////////////////////////////////////////////////////////////////////////////////
// CDXSound is a simple wrapper for a DirectSound object. It is necessary to 
// create a CDXSound object before you can load and play sound using the 
// CDXSoundBuffer object.
//////////////////////////////////////////////////////////////////////////////////
class CDXAPI CDXSound
{
public:
	CDXSound(void);
	~CDXSound(void);

	BOOL Create(void *hWnd);
	BOOL GetCaps(void);
	void SoundOnOff( BOOL sw ) { if( !m_NoSoundCard ) m_SoundOff=sw; }

	BOOL SetMIDIVolume( BYTE rvol, BYTE lvol );
	BOOL SetDIGIVolume( BYTE rvol, BYTE lvol );
	BOOL SetCDVolume( BYTE rvol, BYTE lvol );
	DWORD GetMIDIVolume( void );
	DWORD GetDIGIVolume( void );
	DWORD GetCDVolume( void );

	BOOL m_NoSoundCard, m_SoundOff;
public:
	LPDIRECTSOUND m_lpDS;					// The DirectSound Object
	DSCAPS m_DSCaps;						// Direct sound capabilities
};

//////////////////////////////////////////////////////////////////////////////////
// A simple wrapper for a DirectSoundBuffer object. It is used to load and play 
// WAVE files for the sound effects in your game.
//////////////////////////////////////////////////////////////////////////////////
class CDXAPI CDXSoundBuffer
{
public:
	CDXSoundBuffer( void );
	~CDXSoundBuffer( void );
	void Release( void );

	BOOL Load( CDXSound* pDS, const char* Filename, int Num = 1 );
	BOOL Load( CDXSound* pDS, FILE* fptr, int Num = 1 );
	BOOL LoadFromRes( CDXSound* pDS, const char* Filename, HMODULE hModul = NULL, int Num = 1);
	BOOL Extract( CDXSound* pDS, char *ptr, int Num = 1 );
	BOOL Play( DWORD dwFlags = 0, int Pan = 0, int Volume = 0 );
	BOOL PlayFrequency( DWORD freq, int pan = 0, DWORD dwFlags = 0 );
	BOOL Stop( void );
	void SetVolume( LONG Volume );
	void SetPan( LONG Pan );
	DWORD Playing( void );

	BOOL LoadFromFile( void );
	BOOL LoadFromMemory( char *ptr );
	BOOL CreateSoundBuffer( DWORD dwBufSize, DWORD dwFreq, DWORD dwBitsPerSample,
												 DWORD dwBlkAlign, BOOL bStereo );
	BOOL ReadData( FILE* fp, DWORD dwSize, DWORD dwPos );
	BOOL ReadDataFromMemory( char *ptr, DWORD dwSize );
	LPDIRECTSOUNDBUFFER GetFreeBuffer( void );

public:
	int m_nBuffers;							// The number of sound buffers created
	int m_Current;							// The currently selected buffer
	const char* m_Filename;					// The name of the WAVE file
	CDXSound* m_pDS;						// Contains a pointer to the CDXSound object
	LPDIRECTSOUNDBUFFER *m_lpDSB;			// The IDirectSoundBuffer object
};

#endif
