///////////////////////////////////////////////////////////////////////////////////////////////////
// This Alpha-blending Sample was done by "Niki" for the PPEE-Homepage (http://cust.nol.at/ppee) //
///////////////////////////////////////////////////////////////////////////////////////////////////
#define STRICT
#define WIN32_LEAN_AND_MEAN
#include "stdafx.h"
#include "AlphaBlitter.h"

/////////////////////////////////////////////////////////////////////////////////////////
// 16 bpp alpha blitter
/////////////////////////////////////////////////////////////////////////////////////////

class CAlphaBlitter16 : public CAlphaBlitter
{
        public:
                virtual HRESULT Initialize(DDPIXELFORMAT& r_pixelFormat);

                virtual HRESULT DarkenBlock50(
                        DDSURFACEDESC& r_destSurface,
                        int destX,
                        int destY,
                        VoUInt width,
                        VoUInt height);

                virtual HRESULT AlphaBlt(
                        DDSURFACEDESC& r_destSurface,
                        int destX, 
                        int destY,
                        DDSURFACEDESC& r_srcSurface,
                        RECT * p_srcRect,
                        VoByte alpha);

                virtual HRESULT AlphaBltEx(
                        DDSURFACEDESC& r_destSurface,
                        int destX, 
                        int destY,
                        DDSURFACEDESC& r_srcSurface,
                        RECT * p_srcRect,
                        DDSURFACEDESC& r_alphaSurface);

        private:
                VoUInt32        m_rMask;
                VoUInt32        m_gMask;
                VoUInt32        m_bMask;

                VoUInt32        m_darken50Mask;
};

HRESULT CAlphaBlitter16::Initialize(DDPIXELFORMAT& r_pixelFormat)
{
        m_rMask = r_pixelFormat.dwRBitMask;
        m_gMask = r_pixelFormat.dwGBitMask;
        m_bMask = r_pixelFormat.dwBBitMask;

        // compute darken50 and-mask
        VoUInt32 rMask = m_rMask;
        VoUInt32 gMask = m_gMask;
        VoUInt32 bMask = m_bMask;
        VoUInt32 x;

        x = 1; while ((rMask & x) == 0) x <<= 1; rMask ^= x;
        x = 1; while ((gMask & x) == 0) x <<= 1; gMask ^= x;
        x = 1; while ((bMask & x) == 0) x <<= 1; bMask ^= x;

        m_darken50Mask = rMask | gMask | bMask;

        // success
        return S_OK;
}

HRESULT CAlphaBlitter16::DarkenBlock50(
        DDSURFACEDESC& r_destSurface,
        int destX,
        int destY,
        VoUInt width,
        VoUInt height)
{
        if (width == 0 || height == 0) return S_OK;

        // compute pointers and modulo
        VoUInt16 * p_bits = (VoUInt16 *)&(((VoByte *)r_destSurface.lpSurface)[(destX << 1) + (destY * r_destSurface.lPitch)]);
        VoUInt32 modulo = r_destSurface.lPitch - (width << 1);

        // perform the alpha-blit
        VoUInt32 x;
        VoUInt32 mask = m_darken50Mask;

        do
        {
                x = width;
                do
                {
                        *p_bits++ = VoUInt16((*p_bits & mask) >> 1);
                } while (--x != 0);

                p_bits = (VoUInt16 * )&(((VoByte *)p_bits)[modulo]);

        } while (--height != 0);

        // success
        return S_OK;
}

HRESULT CAlphaBlitter16::AlphaBlt(
        DDSURFACEDESC& r_destSurface,
        int destX, 
        int destY,
        DDSURFACEDESC& r_srcSurface,
        RECT * p_srcRect,
        VoByte alpha)
{
        if (alpha == 0) return S_OK;

        // get blitting dimensions
        int srcX = 0;
        int     srcY = 0;
        VoUInt32 width  = r_srcSurface.dwWidth;
        VoUInt32 height = r_srcSurface.dwHeight;

        if (p_srcRect != NULL)
        {
                width  = p_srcRect->right;
                height = p_srcRect->bottom;
                srcX   = p_srcRect->left;
                srcY   = p_srcRect->top;
        }
        if (width == 0 || height == 0) return S_OK;

        // compute pointers and modulo
        VoUInt16 * p_destBits = (VoUInt16 *)&(((VoByte *)r_destSurface.lpSurface)[(destX << 1) + (destY * r_destSurface.lPitch)]);
        VoUInt16 * p_srcBits  = (VoUInt16 *)&(((VoByte *)r_srcSurface.lpSurface)[(srcX << 1) + (srcY * r_srcSurface.lPitch)]);
        VoUInt32 destModulo = r_destSurface.lPitch - (width << 1);
        VoUInt32 srcModulo  = r_srcSurface.lPitch - (width << 1);

        // perform the alpha-blit
        VoUInt32 x;
        VoUInt32 srcPixel, destPixel;

        VoUInt32 rMask = m_rMask;
        VoUInt32 gMask = m_gMask;
        VoUInt32 bMask = m_bMask;
        VoUInt32 r, g, b;

        do
        {
                x = width;
                do
                {
                        // read the pixel from the source surface
                        srcPixel = *p_srcBits++;

                        // read the pixel from the destination surface
                        destPixel = *p_destBits;

                        // blend color channels
                        r = destPixel & rMask;
                        g = destPixel & gMask;
                        b = destPixel & bMask;

                        destPixel  = (((((srcPixel & rMask) - r) * alpha) >> 8) & rMask) + r;
                        destPixel |= (((((srcPixel & gMask) - g) * alpha) >> 8) & gMask) + g;
                        destPixel |= (((((srcPixel & bMask) - b) * alpha) >> 8) & bMask) + b;

                        // write pixel
                        *p_destBits++ = (VoUInt16)destPixel;

                } while (--x != 0);

                p_srcBits  = (VoUInt16 * )&(((VoByte *)p_srcBits)[srcModulo]);
                p_destBits = (VoUInt16 * )&(((VoByte *)p_destBits)[destModulo]);

        } while (--height != 0);

        // success
        return S_OK;
}

HRESULT CAlphaBlitter16::AlphaBltEx(
        DDSURFACEDESC& r_destSurface,
        int destX, 
        int destY,
        DDSURFACEDESC& r_srcSurface,
        RECT * p_srcRect,
        DDSURFACEDESC& r_alphaSurface)
{
        // get blitting dimensions
        int srcX = 0;
        int     srcY = 0;
        VoUInt32 width  = r_srcSurface.dwWidth;
        VoUInt32 height = r_srcSurface.dwHeight;

        if (p_srcRect != NULL)
        {
                width  = p_srcRect->right;
                height = p_srcRect->bottom;
                srcX   = p_srcRect->left;
                srcY   = p_srcRect->top;
        }
        if (width == 0 || height == 0) return S_OK;

        // compute pointers and modulo
        VoUInt16 * p_destBits  = (VoUInt16 *)&(((VoByte *)r_destSurface.lpSurface)[(destX << 1) + (destY * r_destSurface.lPitch)]);
        VoUInt16 * p_srcBits   = (VoUInt16 *)&(((VoByte *)r_srcSurface.lpSurface)[(srcX << 1) + (srcY * r_srcSurface.lPitch)]);
        VoByte   * p_alphaBits = (VoByte *) &(((VoByte*)r_alphaSurface.lpSurface)[srcX + srcY * r_alphaSurface.lPitch]);
        VoUInt32 destModulo  = r_destSurface.lPitch - (width << 1);
        VoUInt32 srcModulo   = r_srcSurface.lPitch - (width << 1);
        VoUInt32 alphaModulo = r_alphaSurface.lPitch - width;

        // perform the alpha-blit
        VoUInt32 x;
        VoUInt32 srcPixel, destPixel;

        VoUInt32 rMask = m_rMask;
        VoUInt32 gMask = m_gMask;
        VoUInt32 bMask = m_bMask;
        VoUInt32 r, g, b;
        VoUInt32 alpha;

        do
        {
                x = width;
                do
                {
                        // read the pixel from the source surface
                        srcPixel = *p_srcBits++;
                        alpha = *p_alphaBits++;

                        // read the pixel from the destination surface
                        destPixel = *p_destBits;

                        // blend color channels
                        r = destPixel & rMask;
                        g = destPixel & gMask;
                        b = destPixel & bMask;

                        destPixel  = (((((srcPixel & rMask) - r) * alpha) >> 8) & rMask) + r;
                        destPixel |= (((((srcPixel & gMask) - g) * alpha) >> 8) & gMask) + g;
                        destPixel |= (((((srcPixel & bMask) - b) * alpha) >> 8) & bMask) + b;

                        // write pixel
                        *p_destBits++ = (VoUInt16)destPixel;

                } while (--x != 0);

                p_srcBits   = (VoUInt16 * )&(((VoByte *)p_srcBits)[srcModulo]);
                p_destBits  = (VoUInt16 * )&(((VoByte *)p_destBits)[destModulo]);
                p_alphaBits = p_alphaBits + alphaModulo;

        } while (--height != 0);

        // success
        return S_OK;
}

/////////////////////////////////////////////////////////////////////////////////////////
// 24 bpp alpha blitter
/////////////////////////////////////////////////////////////////////////////////////////

// TODO: Sorry, but I don't have a graphics card that supports 24 bpp modes. However,
//       the implementation should be straight forward.

/////////////////////////////////////////////////////////////////////////////////////////
// 32 bpp alpha blitter
/////////////////////////////////////////////////////////////////////////////////////////

class CAlphaBlitter32 : public CAlphaBlitter
{
        public:
                virtual HRESULT Initialize(DDPIXELFORMAT& r_pixelFormat);

                virtual HRESULT DarkenBlock50(
                        DDSURFACEDESC& r_destSurface,
                        int destX,
                        int destY,
                        VoUInt width,
                        VoUInt height);

                virtual HRESULT AlphaBlt(
                        DDSURFACEDESC& r_destSurface,
                        int destX, 
                        int destY,
                        DDSURFACEDESC& r_srcSurface,
                        RECT * p_srcRect,
                        VoByte alpha);

                virtual HRESULT AlphaBltEx(
                        DDSURFACEDESC& r_destSurface,
                        int destX, 
                        int destY,
                        DDSURFACEDESC& r_srcSurface,
                        RECT * p_srcRect,
                        DDSURFACEDESC& r_alphaSurface);

        private:
                VoUInt32        m_rMask;
                VoUInt32        m_gMask;
                VoUInt32        m_bMask;
                VoUInt32        m_darken50Mask;
};

HRESULT CAlphaBlitter32::Initialize(DDPIXELFORMAT& r_pixelFormat)
{
        m_rMask = r_pixelFormat.dwRBitMask;
        m_gMask = r_pixelFormat.dwGBitMask;
        m_bMask = r_pixelFormat.dwBBitMask;

        // compute darken50 and-mask
        VoUInt32 rMask = m_rMask;
        VoUInt32 gMask = m_gMask;
        VoUInt32 bMask = m_bMask;
        VoUInt32 x;

        x = 1; while ((rMask & x) == 0) x <<= 1; rMask = rMask ^ x;
        x = 1; while ((gMask & x) == 0) x <<= 1; gMask = gMask ^ x;
        x = 1; while ((bMask & x) == 0) x <<= 1; bMask = bMask ^ x;

        m_darken50Mask = rMask | gMask | bMask;

        // success
        return S_OK;
}

HRESULT CAlphaBlitter32::DarkenBlock50(
        DDSURFACEDESC& r_destSurface,
        int destX,
        int destY,
        VoUInt width,
        VoUInt height)
{
        if (width == 0 || height == 0) return S_OK;

        // compute pointers and modulo
        VoUInt32 * p_bits = (VoUInt32 *)&(((VoByte *)r_destSurface.lpSurface)[(destX << 2) + (destY * r_destSurface.lPitch)]);
        VoUInt32 modulo = r_destSurface.lPitch - (width << 2);

        // perform the alpha-blit
        VoUInt32 x;
        VoUInt32 mask = m_darken50Mask;

        do
        {
                x = width;
                do
                {
                        *p_bits++ = (*p_bits & mask) >> 1;
                } while (--x != 0);

                p_bits = (VoUInt32 * )&(((VoByte *)p_bits)[modulo]);

        } while (--height != 0);

        // success
        return S_OK;
}

HRESULT CAlphaBlitter32::AlphaBlt(
        DDSURFACEDESC& r_destSurface,
        int destX, 
        int destY,
        DDSURFACEDESC& r_srcSurface,
        RECT * p_srcRect,
        VoByte alpha)
{
        if (alpha == 0) return S_OK;

        // get blitting dimensions
        int srcX = 0;
        int     srcY = 0;
        VoUInt32 width  = r_srcSurface.dwWidth;
        VoUInt32 height = r_srcSurface.dwHeight;

        if (p_srcRect != NULL)
        {
                width  = p_srcRect->right;
                height = p_srcRect->bottom;
                srcX   = p_srcRect->left;
                srcY   = p_srcRect->top;
        }
        if (width == 0 || height == 0) return S_OK;

        // compute pointers and modulo
        VoUInt32 * p_destBits = (VoUInt32 *)&(((VoByte *)r_destSurface.lpSurface)[(destX << 2) + (destY * r_destSurface.lPitch)]);
        VoUInt32 * p_srcBits  = (VoUInt32 *)&(((VoByte *)r_srcSurface.lpSurface)[(srcX << 2) + (srcY * r_srcSurface.lPitch)]);
        VoUInt32 destModulo = r_destSurface.lPitch - (width << 2);
        VoUInt32 srcModulo  = r_srcSurface.lPitch - (width << 2);

        // perform the alpha-blit
        VoUInt32 x;
        VoUInt32 srcPixel, destPixel;

        VoUInt32 rMask = m_rMask;
        VoUInt32 gMask = m_gMask;
        VoUInt32 bMask = m_bMask;
        VoUInt32 r, g, b;

        do
        {
                x = width;
                do
                {
                        // read the pixel from the source surface
                        srcPixel = *p_srcBits++;

                        // read the pixel from the destination surface
                        destPixel = *p_destBits;

                        // blend color channels
                        r = destPixel & rMask;
                        g = destPixel & gMask;
                        b = destPixel & bMask;

                        destPixel  = (((((srcPixel & rMask) - r) * alpha) >> 8) & rMask) + r;
                        destPixel |= (((((srcPixel & gMask) - g) * alpha) >> 8) & gMask) + g;
                        destPixel |= (((((srcPixel & bMask) - b) * alpha) >> 8) & bMask) + b;

                        // write pixel
                        *p_destBits++ = (VoUInt32)destPixel;

                } while (--x != 0);

                p_srcBits  = (VoUInt32 * )&(((VoByte *)p_srcBits)[srcModulo]);
                p_destBits = (VoUInt32 * )&(((VoByte *)p_destBits)[destModulo]);

        } while (--height != 0);

        // success
        return S_OK;
}

HRESULT CAlphaBlitter32::AlphaBltEx(
        DDSURFACEDESC& r_destSurface,
        int destX, 
        int destY,
        DDSURFACEDESC& r_srcSurface,
        RECT * p_srcRect,
        DDSURFACEDESC& r_alphaSurface)
{
        // get blitting dimensions
        int srcX = 0;
        int     srcY = 0;
        VoUInt32 width  = r_srcSurface.dwWidth;
        VoUInt32 height = r_srcSurface.dwHeight;

        if (p_srcRect != NULL)
        {
                width  = p_srcRect->right;
                height = p_srcRect->bottom;
                srcX   = p_srcRect->left;
                srcY   = p_srcRect->top;
        }
        if (width == 0 || height == 0) return S_OK;

        // compute pointers and modulo
        VoUInt32 * p_destBits  = (VoUInt32 *)&(((VoByte *)r_destSurface.lpSurface)[(destX << 2) + (destY * r_destSurface.lPitch)]);
        VoUInt32 * p_srcBits   = (VoUInt32 *)&(((VoByte *)r_srcSurface.lpSurface)[(srcX << 2) + (srcY * r_srcSurface.lPitch)]);
        VoByte   * p_alphaBits = (VoByte *) &(((VoByte*)r_alphaSurface.lpSurface)[srcX + srcY * r_alphaSurface.lPitch]);
        VoUInt32 destModulo  = r_destSurface.lPitch - (width << 2);
        VoUInt32 srcModulo   = r_srcSurface.lPitch - (width << 2);
        VoUInt32 alphaModulo = r_alphaSurface.lPitch - width;

        // perform the alpha-blit
        VoUInt32 x;
        VoUInt32 srcPixel, destPixel;

        VoUInt32 rMask = m_rMask;
        VoUInt32 gMask = m_gMask;
        VoUInt32 bMask = m_bMask;
        VoUInt32 r, g, b;
        VoUInt32 alpha;

        do
        {
                x = width;
                do
                {
                        // read the pixel from the source surface
                        srcPixel = *p_srcBits++;
                        alpha = *p_alphaBits++;

                        // read the pixel from the destination surface
                        destPixel = *p_destBits;

                        // blend color channels
                        r = destPixel & rMask;
                        g = destPixel & gMask;
                        b = destPixel & bMask;

                        destPixel  = (((((srcPixel & rMask) - r) * alpha) >> 8) & rMask) + r;
                        destPixel |= (((((srcPixel & gMask) - g) * alpha) >> 8) & gMask) + g;
                        destPixel |= (((((srcPixel & bMask) - b) * alpha) >> 8) & bMask) + b;

                        // write pixel
                        *p_destBits++ = (VoUInt32)destPixel;

                } while (--x != 0);

                p_srcBits   = (VoUInt32 * )&(((VoByte *)p_srcBits)[srcModulo]);
                p_destBits  = (VoUInt32 * )&(((VoByte *)p_destBits)[destModulo]);
                p_alphaBits = p_alphaBits + alphaModulo;

        } while (--height != 0);

        // success
        return S_OK;
}

/////////////////////////////////////////////////////////////////////////////////////////
// API
/////////////////////////////////////////////////////////////////////////////////////////

CAlphaBlitter * NewAlphaBlitter(DDPIXELFORMAT& r_pixelFormat)
{
        switch (r_pixelFormat.dwRGBBitCount)
        {
                case 16:
                        CAlphaBlitter16 * p_blitter16;
                        p_blitter16 = new CAlphaBlitter16();
                        if (p_blitter16 == NULL) return NULL;
                        p_blitter16->Initialize(r_pixelFormat);
                        return p_blitter16;

                case 32:
                        CAlphaBlitter32 * p_blitter32;
                        p_blitter32 = new CAlphaBlitter32();
                        if (p_blitter32 == NULL) return NULL;
                        p_blitter32->Initialize(r_pixelFormat);
                        return p_blitter32;
        }
        return NULL;
}
