//////////////////////////////////////////////////////////////////////////////////
// $Source: /usr/cvsroot/cdx/src/cdx/cdx.h,v $
// $Author: MICHAELR $
//
// $Log: cdx.h,v $
// Revision 2.14  1999/08/30 05:25:07  MICHAELR
// Made CDXLog related changes
//
// Revision 2.13  1999/08/29 22:29:04  MICHAELR
// added CDXDLL support to classes/etc
//
// Revision 2.12  1999/08/08 15:24:33  mseibert
// Added CDXErrorToString to return more informational error messages
//
// Revision 2.11  1999/07/17 09:16:27  Mindcry
// Added CDXPacker to CDXINCLUDEALL
//
// Revision 2.10  1999/07/12 14:38:34  janni
// fixed problem with CDXError did not compile
//
// Revision 2.9  1999/07/12 13:38:43  bsimser
// Updated CDXError to reflect the new parameters.
//
// Revision 2.8  1999/07/12 09:15:28  janni
// added CDXError function which display a general error
// message box and quits the program
//
// Revision 2.7  1999/05/21 14:20:31  MICHAELR
// Changed DI and DS versions to 0x0500
//
// Revision 2.6  1999/05/20 15:29:02  pietro
// Multiple changes:
// * fixed #include bugs in all .cpp and various .h files
// * fixed all rcsid[] bugs
// * added conditional compile variable CDXINCLUDEALL - when defined,
//   all #include files are included in cdx.h to keep backward compatibility
// * All the libraries are created in ..\..\lib\vc\ directory, library names are
//   cdx.lib/cdxd.lib/cdxdx3.lib/cdxdx3d.lib/cdxadx3.lib/cdxadx3d.lib
//
// Revision 2.5  1999/05/19 14:02:54  bsimser
// Removed includes of other base classes.
//
// Revision 2.4  1999/05/11 18:55:34  MICHAELR
// Renamed macro DELETE to SAFEDELETE to resolve a duplication problem with WINNT.H
//
// Revision 2.3  1999/05/11 04:13:48  bsimser
// Added the DELETE macro (similar to RELEASE) for objects that don't have
// a release method.
//
// Revision 2.2  1999/05/03 20:40:33  MICHAELR
// The CDX_* defines where not 'defined'
// DIRECTDRAW_VERSION was incorrectly called DIRECTDRAWVERSION
// The *_VERSION were all wrong, they were all missing a 0x0XXX, e.g. 0x300 should have been 0x0300
// DX3 compatible DDCreateSurface(...) was using IDirectDraw4
//
// Revision 2.1  1999/05/02 23:18:20  pietro
// added CDXFile class as file operation wrapper.
// the error handling is not yet working wery well - we need to create some
// general error handling routines.
//
// Revision 2.0  1999/05/01 13:51:16  bsimser
// Updated revision number to 2.0
//
// Revision 1.1.1.1  1999/05/01 04:10:56  bsimser
// Initial revision to cvs
//
// $Revision: 2.14 $
//////////////////////////////////////////////////////////////////////////////////
#ifndef CDX_H
#define CDX_H

// the following section if for building CDX as a DLL

// Begin CDX DLL section ////////////////////////////////////////////

// note _CDX_BUILDDLL & _CDX_USEDLL should NOT be used together
#if defined(_CDX_BUILDDLL) && defined(_CDX_USEDLL)
#error Sorry, you cannot use _CDX_BUILDDLL & _CDX_USEDLL together
#endif

// this is initially defined as nothing
#define CDXAPI

// this should be specified to make CDX as a DLL
#ifdef _CDX_BUILDDLL
#undef CDXAPI
#define CDXAPI __declspec(dllexport)
#endif

// the following should be specified if the user wants to
// use cdx.dll.  Otherwise chaos ensues when you try to delete objects
#ifdef _CDX_USEDLL
#undef CDXAPI
#define CDXAPI __declspec(dllimport)
#endif
// End CDX DLL section //////////////////////////////////////////////

// If you want to support DX running under NT 4.0 (currently only 
// supporting DX3) then define NTDX3 in your program
#ifdef NTDX3

#define DIRECTDRAW_VERSION		0x0300
#define DIRECTINPUT_VERSION	0x0300
#define DIRECTSOUND_VERSION	0x0300

#else

// CDX currently supports DX6
#define DIRECTDRAW_VERSION		0x0500
#define DIRECTINPUT_VERSION	0x0500
#define DIRECTSOUND_VERSION	0x0500

#endif

// CDX Currently support DirectX 6
#define CDX_DDVER		0x0600
#define CDX_DIVER		0x0500
#define CDX_DSVER		0x0500


// System includes
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <stdio.h>

// DirectX includes
#include <ddraw.h>
#include <dinput.h>
#include <dsound.h>
#include <dplay.h>
#include <dplobby.h>

//#include "cdxlog.h"

    
    // Janni, had to move it here, because Bil changed CDXError to use CDXScreen
// otherwise the lib did not compile



typedef struct
{
    RGBQUAD depth;		// How many bits of precision per channel
    RGBQUAD Position;	// At what bit position does the color start
	UINT bpp;
} RGBFORMAT;


//////////////////////////////////////////////////////////////////////////////////
// For those lazy coders, #define CDXINCLUDEALL and all cdx headers will get 
// included. Otherwise, you need to #include all header files you will need 
// (depending on modules you are going to use)
//////////////////////////////////////////////////////////////////////////////////
	#include "cdxscreen.h"
	#include "cdxsurface.h"
	#include "cdxsound.h"
	#include "cdxinput.h"
	#include "cdxlink.h"



//////////////////////////////////////////////////////////////////////////////////
// Globals
//////////////////////////////////////////////////////////////////////////////////
extern "C"
{
	// CDX Functions
	extern CDXAPI void Clip(int *DestX, int *DestY, RECT *SrcRect, RECT *DestRect);

	// DDUTILS Functions
#if DIRECTDRAW_VERSION >= CDX_DDVER
	extern CDXAPI IDirectDrawPalette* DDLoadPalette(IDirectDraw4 *pdd, LPCSTR szBitmap);
	extern CDXAPI IDirectDrawSurface4* DDLoadBitmap(IDirectDraw4 *pdd, LPCSTR szBitmap, int dx, int dy, BOOL memoryType);
	extern CDXAPI IDirectDrawSurface4* DDLoadSizeBitmap(IDirectDraw4 *pdd, LPCSTR szBitmap, int *dx, int *dy, BOOL memoryType);
	IDirectDrawSurface4 *DDCreateSurface(IDirectDraw4 *pdd, DWORD width, DWORD height, DDSURFACEDESC2 *ddsd);
	extern HRESULT             DDReLoadBitmap(IDirectDrawSurface4 *pdds, LPCSTR szBitmap);
	extern HRESULT             DDCopyBitmap(IDirectDrawSurface4 *pdds, HBITMAP hbm, int x, int y, int dx, int dy);
	extern DWORD               DDColorMatch(IDirectDrawSurface4 *pdds, COLORREF rgb);
	extern HRESULT             DDSetColorKey(IDirectDrawSurface4 *pdds, COLORREF rgb);
    extern BOOL                GetRGBFormat(IDirectDrawSurface4* Surface, RGBFORMAT* rgb);
#else
	extern CDXAPI IDirectDrawPalette* DDLoadPalette(IDirectDraw *pdd, LPCSTR szBitmap);
	extern CDXAPI IDirectDrawSurface* DDLoadBitmap(IDirectDraw *pdd, LPCSTR szBitmap, int dx, int dy, BOOL memoryType);
	extern CDXAPI IDirectDrawSurface* DDLoadSizeBitmap(IDirectDraw *pdd, LPCSTR szBitmap, int *dx, int *dy, BOOL memoryType);
	IDirectDrawSurface *DDCreateSurface(IDirectDraw *pdd, DWORD width, DWORD height, DDSURFACEDESC *ddsd);
	extern CDXAPI HRESULT             DDReLoadBitmap(IDirectDrawSurface *pdds, LPCSTR szBitmap);
	extern CDXAPI HRESULT             DDCopyBitmap(IDirectDrawSurface *pdds, HBITMAP hbm, int x, int y, int dx, int dy);
	extern CDXAPI DWORD               DDColorMatch(IDirectDrawSurface *pdds, COLORREF rgb);
	extern CDXAPI HRESULT             DDSetColorKey(IDirectDrawSurface *pdds, COLORREF rgb);
    extern CDXAPI BOOL                GetRGBFormat(IDirectDrawSurface* Surface, RGBFORMAT* rgb);
#endif

	extern BOOL __cdecl        IsMMX(void);

	// DirectX Errors
	extern CDXAPI void DDError(HRESULT hErr, void* hWnd, LPCSTR szFile, int iLine);
	extern CDXAPI void CDXError( CDXScreen *Screen, char *str, ...);
	extern CDXAPI void DSError(HRESULT hErr, void* hWnd);
	extern CDXAPI char* DXErrorToString( HRESULT hErr );
}

//////////////////////////////////////////////////////////////////////////////////
// MACROS
//////////////////////////////////////////////////////////////////////////////////
#define SAFEDELETE(x) if(x != NULL) { delete x; x = NULL; }
#define DELETEARRAY(x) if(x != NULL) { delete [] x; x = NULL; }
#define RELEASE(x) if(x != NULL) { x->Release(); x = NULL; }
#define ODS(a) OutputDebugString(a)

#endif
