//////////////////////////////////////////////////////////////////////////////////
// $Source: /usr/cvsroot/cdx/src/cdx/cdxscreen.h,v $
// $Author: jhebert $
//
// $Log: cdxscreen.h,v $
// Revision 2.19  1999/10/17 19:57:49  jhebert
// - Fixed the restore function for windowed mode.
// - Fixed the delay parameter in the 8bpp Fade functions.  Values between 0-50 are good.
// - Added Hi-color fade in/out functions.  See headers for details.
// - Added an additional parameter to the Flip function, displayFPS. If this is set to
//   TRUE the framerate will be displayed in the upper right hand corner.
//
// Revision 2.18  1999/10/17 06:12:44  janni
// hopefully fixed compile errors in MSVC 5
//
// Revision 2.17  1999/10/08 10:41:14  janni
// Added SurfaceList to CDX, CDX now keeps track of every surface in the SurfaceList member
// When you delete the CDXScreen object, it automatically frees all remaining surfaces
// I added the functions AddSurfaceToList,ReleaseSurfaceFromList and ReleaseAllSurfaces
// I added the public member SurfaceList
//
// Revision 2.16  1999/08/29 22:29:04  MICHAELR
// added CDXDLL support to classes/etc
//
// Revision 2.15  1999/08/18 14:55:47  Mindcry
// Added information on the HEL of the enumerated DD Drivers
//
// Revision 2.14  1999/08/17 19:53:10  Mindcry
// fixed the problem with the missing MAX_DDDEVICEID_STRING in some DDRAW headers
//
// Revision 2.13  1999/08/17 18:56:39  Mindcry
// WinNT fix ( i hope )
//
// Revision 2.12  1999/08/14 15:07:49  Mindcry
// added support for multimontiring and/or use of 3dgfx card new function:
// SelectDriver, EnumVideoModes, EnumDrivers
//
// Revision 2.11  1999/08/01 18:30:48  MICHAELR
// Added param to Flip method to allow flipping in windowed mode without stretching
//
// Revision 2.10  1999/07/25 17:11:53  janni
// added MakeColor function to convert colors
//
// Revision 2.9  1999/05/28 07:53:50  janni
// added CreateBackBufferInVRAM function
// rewrote CreateSurface(CreateWIndowed to support backbuffers in sysram
//
// Revision 2.8  1999/05/21 06:59:47  janni
// took back the changes from revision 2.7
// removed AdjustWinStyle
// only SetVideoMode is in cdxscreen now
//
// Revision 2.7  1999/05/20 22:24:27  janni
// added SetVideoMode which is a shell for CreateWindowed/CreateFullScreen
// added AdjustWinStyle, moved from CDX App Wizard code to CDXScreen, it is called automatically now when you set a videomode
//
// Revision 2.6  1999/05/14 16:30:56  janni
// fixed BOOL in definition of CDXScreen::Flip into bool to provide Visual C 5 compatibility
//
// Revision 2.5  1999/05/14 08:50:01  janni
// added VSync parameter to CDXScreen:Flip , default = true
// true means flip with vsync, flase means flip without vsync
// this works in windowed and fullscreen modes!
//
// Revision 2.4  1999/05/07 20:25:55  janni
// added SetTripleBuffering and GetTripleBuffering
// added WaitForVerticalBlank
//
// Revision 2.3  1999/05/06 14:11:21  janni
// Janni: added
//    members:
//     CDX_VIDEOMODESSTRUCT;
//     m_NumberOfVideoModes;
//     m_VideoModes;
//
//     functions:
//     EnumDisplayModesCallback
//     GetNumberOfVideoModes
//     GetVideoModeInformation
//     CheckIfVideoModeExists
//
// Revision 2.2  1999/05/03 21:08:56  MICHAELR
// fixed a place i missed where DIRECTDRAW_VERSION was incorrectly called DIRECTDRAWVERSION
//
// Revision 2.1  1999/05/03 20:46:03  MICHAELR
// DIRECTDRAW_VERSION was incorrectly called DIRECTDRAWVERSION
// CDXScreen::m_lpDD didn't support DX3 compaible LPDIRECTDRAW
//
// Revision 2.0  1999/05/01 13:51:16  bsimser
// Updated revision number to 2.0
//
// Revision 1.1.1.1  1999/05/01 04:10:56  bsimser
// Initial revision to cvs
//
// $Revision: 2.19 $
//////////////////////////////////////////////////////////////////////////////////
#ifndef CDXSCREEN_H
#define CDXSCREEN_H

// double linked list from STL
//#include <vector>

// structure for videomodes
typedef struct 
{
    DWORD					Width;
    DWORD					Height;
    DWORD					BPP;
    void				*   NextMode;
} CDX_VIDEOMODESSTRUCT;

// fix for those strange DDRAW header files where it miss
#ifndef MAX_DDDEVICEID_STRING
	#define MAX_DDDEVICEID_STRING 512
#endif

// structure for direct draw drivers
struct CDX_DRIVERSTRUCT
{
	LPGUID	lpGUID;
	GUID	guid;
	char	lpName[MAX_DDDEVICEID_STRING],
		lpDesc[MAX_DDDEVICEID_STRING];
	DDCAPS	DDcaps, HELcaps;
#if DIRECTDRAW_VERSION >= CDX_DDVER
	HMONITOR hMonitor;
#endif
	CDX_DRIVERSTRUCT *NextDriver;
};
		
// callback function for video mode enumeration, implemented in ddutils.cpp
#if DIRECTDRAW_VERSION >= CDX_DDVER
HRESULT CALLBACK EnumDisplayModesCallback( LPDDSURFACEDESC2 lpDDSurfaceDesc, LPVOID lpContext  );
#else
HRESULT CALLBACK EnumDisplayModesCallback( LPDDSURFACEDESC lpDDSurfaceDesc, LPVOID lpContext  );
#endif

// callback function for devices enumeration, implemented in ddutils.cpp
#if DIRECTDRAW_VERSION >= CDX_DDVER
BOOL WINAPI DDEnumCallback( GUID FAR *lpGUID, LPSTR lpDesc, LPSTR lpName, LPVOID lpContext, HMONITOR hm );
#else
BOOL WINAPI DDEnumCallback( GUID FAR *lpGUID, LPSTR lpDesc, LPSTR lpName, LPVOID lpContext );
#endif

// Forward declaration needed for the CDXScreen class
class CDXAPI CDXSurface;

//////////////////////////////////////////////////////////////////////////////////
// CDXScreen is the primary object of the library and every program that
// uses CDX must include a CDXScreen object.  As the name implies CDXScreen
// contains all the functions necessary to control the video display,
// currently this also includes functions to control the palette.
//////////////////////////////////////////////////////////////////////////////////
class CDXAPI CDXScreen
{
public:
	CDXScreen();
	CDXScreen(void *hWnd, DWORD Width, DWORD Height, DWORD BPP, BOOL bVGA = FALSE);
	CDXScreen(void *hWnd, int Width, int Height);
	~CDXScreen();

	BOOL CreateFullScreen(void *hWnd, DWORD Width, DWORD Height, DWORD BPP, BOOL bVGA = FALSE);
	BOOL CreateWindowed(void *hWnd, int Width, int Height);
	BOOL LoadBitmap(const char* szFilename);
	BOOL LoadPalette(const char* szFilename);
	void Fill(DWORD FillColor);
	void Restore(void);
	HRESULT Flip(BOOL VSync=TRUE, BOOL FlipWithStretch=TRUE, BOOL displayFPS = FALSE);

	void SetColor(int col, int r, int g, int b);
	void GetColor(int col, int *r, int *g, int *b);
	void SetPalette(int Start, int Count, LPPALETTEENTRY lpPE);
	void GetPalette(int Start, int Count, LPPALETTEENTRY lpPE);
	void FillPalette(int r, int g, int b);
	void GreyScale(void);
	void FadeIn(int delay, LPPALETTEENTRY lpPE);
	void FadeOut(int delay);
	void FadeTo(int r, int g, int b, int delay);

	int GetWidth(void) { return m_dwPixelWidth; }
	int GetHeight(void) { return m_dwPixelHeight; }
	int GetBPP(void) { return m_BPP; }

#if DIRECTDRAW_VERSION >= CDX_DDVER
	LPDIRECTDRAW4 GetDD(void) { return m_lpDD; }
#else
	LPDIRECTDRAW GetDD(void) { return m_lpDD; }
#endif

	CDXSurface* GetFront(void) { return m_lpDDSFront; }
	CDXSurface* GetBack(void) { return m_lpDDSBack; }
	LPDIRECTDRAWPALETTE GetPalette(void) { return m_lpDDPalette; }
// Janni
    DWORD GetNumberOfVideoModes( void );
    BOOL  GetVideoModeInformation( DWORD Index  , DWORD * Width , DWORD * Height , DWORD * BPP );
    BOOL  CheckIfVideoModeExists( DWORD Width , DWORD Height , DWORD BPP );
	HRESULT WaitForVerticalBlank( void );
	void InitCDXScreen();
	void CreateBackBufferInVRAM( BOOL Flag ) { m_BackBufferInVRAM = Flag; };
	void SetTripleBuffering( BOOL Flag ) { m_TripleBuffering = Flag; }
	BOOL GetTripleBuffering( void ) { return m_TripleBuffering; }
	BOOL SetVideoMode( void * hWnd , int Width , int Height , int BPP = 0 );
    DWORD MakeColor( DWORD Color , CDXSurface * Surface = NULL );
	void AddSurfaceToList( CDXSurface * Surface );
	void ReleaseAllSurfaces( void );
	void ReleaseSurfaceFromList( CDXSurface * Surface );
// Janni
	BOOL SelectDriver( DWORD Index );
	void FadeToBlack(WORD duration);
	void FadeToSource(WORD duration);

private:
	BOOL EnumVideoModes( void );
	BOOL EnumDrivers( void );

public:
	// Beda
	HRESULT FlipToGDI();

#if DIRECTDRAW_VERSION >= CDX_DDVER
	LPDIRECTDRAW4 m_lpDD;						 // the DirectDraw object
#else
	LPDIRECTDRAW m_lpDD;						 // the DirectDraw object
#endif

	CDXSurface* m_lpDDSFront;					 // a CDXSurface object for the front buffer
	CDXSurface* m_lpDDSBack;					 // a CDXSurface object for the back buffer

#if DIRECTDRAW_VERSION >= CDX_DDVER
	LPDIRECTDRAWSURFACE4 m_ZBuffer;		
#else 
	LPDIRECTDRAWSURFACE  m_ZBuffer;		
#endif

	LPDIRECTDRAWPALETTE m_lpDDPalette;			 // the DirectDraw palette object
	LPDIRECTDRAWCLIPPER m_lpClipper;			 // the DirectDraw clipper, used for window mode only

	DWORD m_dwPixelWidth;						 // the screen width, in pixels
	DWORD m_dwPixelHeight;						 // the screen height, in pixels
	DWORD m_BPP;								 // bits per pixel (colour depth)
	void* m_hWnd;								 // pointer to the HWND handle of the window

	BOOL m_bFullScreen;							 // is the program running in fullscreen mode?
// Janni
    DWORD                   m_NumberOfVideoModes; // number of video modes the video adapter supports
    CDX_VIDEOMODESSTRUCT *  m_VideoModes;		// linked list with all video modes informations
	BOOL					m_TripleBuffering;	// is triple buffering enabled ?
	BOOL					m_BackBufferInVRAM;
	//std::vector<CDXSurface *> SurfaceList;	// list which holds every created surface
// Janni

    DWORD             m_NumberOfDrivers; // number of grafikcards installed
    CDX_DRIVERSTRUCT *m_Drivers;         // linked list with grafikcard informations
    DWORD             m_SelectedDriver;  // Which driver will be used for creating DirectDraw

private:
	DWORD m_frameCount;
	BOOL  m_GDIVisible; // for Flip to GDI  // Beda
	DWORD m_accumulatedFrameTime;
	DWORD m_frameStartTime;
	LONG  m_fps;
};

#endif
